@@ -38,7 +38,7 @@
  *
  * Martin Schulze's patches adapted to Util-Linux by Nicolai Langfeldt.
  *
- * 1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+ * 1999-02-22 Arkadiusz Mikiewicz <misiek@pld.ORG.PL>
  * - added Native Language Support
  * Sun Mar 21 1999 - Arnaldo Carvalho de Melo <acme@conectiva.com.br>
  * - fixed strerr(errno) in gettext calls
@@ -64,8 +64,13 @@ static char version_string[] = "vipw 1.4";
 #include <unistd.h>
 
 #include "setpwnam.h"
-#include "xstrncpy.h"
+#include "strutils.h"
 #include "nls.h"
+#include "c.h"
+
+#ifdef HAVE_LIBSELINUX
+#include <selinux/selinux.h>
+#endif
 
 #define FILENAMELEN 67
 
@@ -82,7 +87,7 @@ static void
 copyfile(int from, int to) {
 	int nr, nw, off;
 	char buf[8*1024];
-	
+
 	while ((nr = read(from, buf, sizeof(buf))) > 0)
 		for (off = 0; off < nr; nr -= nw, off += nw)
 			if ((nw = write(to, buf + off, nr)) < 0)
@@ -127,7 +132,7 @@ static int
 pw_lock(void) {
 	int lockfd, fd, ret;
 
-	/* 
+	/*
 	 * If the password file doesn't exist, the system is hosed.
 	 * Might as well try to build one.  Set the close-on-exec bit so
 	 * that users can't get at the encrypted passwords while editing.
@@ -135,23 +140,24 @@ pw_lock(void) {
 	 */
 #if 0 /* flock()ing is superfluous here, with the ptmp/ptmptmp system. */
 	if (flock(lockfd, LOCK_EX|LOCK_NB)) {
-		(void)fprintf(stderr,
-		    _("%s: the %s file is busy.\n"), progname, 
-			      program == VIPW ? "password" : "group" );
+		if (program == VIPW)
+			fprintf(stderr, _("%s: the password file is busy.\n"),
+				progname);
+		else
+			fprintf(stderr, _("%s: the group file is busy.\n"),
+				progname);
 		exit(1);
 	}
 #endif
 
-	if ((fd = open(tmptmp_file, O_WRONLY|O_CREAT, 0644)) == -1) {
-	  (void)fprintf(stderr,
-		   "%s: %s: %s\n", progname, tmptmp_file, strerror(errno));
-	  exit(1);
-	}
+	if ((fd = open(tmptmp_file, O_WRONLY|O_CREAT, 0600)) == -1)
+	    err(EXIT_FAILURE, _("%s: open failed"), tmptmp_file);
+
 	ret = link(tmptmp_file, tmp_file);
 	(void)unlink(tmptmp_file);
 	if (ret == -1) {
 	    if (errno == EEXIST)
-		(void)fprintf(stderr, 
+		(void)fprintf(stderr,
 			      _("%s: the %s file is busy (%s present)\n"),
 			      progname,
 			      program == VIPW ? "password" : "group",
@@ -161,7 +167,7 @@ pw_lock(void) {
 		(void)fprintf(stderr, _("%s: can't link %s: %s\n"), progname,
 			      tmp_file, strerror(errsv));
 	    }
-	    exit(1);
+	    exit(EXIT_FAILURE);
 	}
 
 	lockfd = open(orig_file, O_RDONLY, 0);
@@ -170,30 +176,50 @@ pw_lock(void) {
 		(void)fprintf(stderr, "%s: %s: %s\n",
 		    progname, orig_file, strerror(errno));
 		unlink(tmp_file);
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 	copyfile(lockfd, fd);
 	(void)close(lockfd);
 	(void)close(fd);
-	return(1);
+	return 1;
 }
 
 static void
 pw_unlock(void) {
-  char tmp[FILENAMELEN];
+	char tmp[FILENAMELEN+4];
   
-  sprintf(tmp, "%s%s", orig_file, ".OLD");
-  unlink(tmp);
-  link(orig_file, tmp);
-  if (rename(tmp_file, orig_file) == -1) {
-    int errsv = errno;
-    (void)fprintf(stderr, 
-		  _("%s: can't unlock %s: %s (your changes are still in %s)\n"), 
-		  progname, orig_file, strerror(errsv), tmp_file);
-    exit(1);
-  }
-  (void)unlink(tmp_file);
+	sprintf(tmp, "%s%s", orig_file, ".OLD");
+	unlink(tmp);
+
+	if (link(orig_file, tmp))
+		warn(_("%s: create a link to %s failed"), orig_file, tmp);
+
+#ifdef HAVE_LIBSELINUX
+	if (is_selinux_enabled() > 0) {
+	  security_context_t passwd_context=NULL;
+	  int ret=0;
+	  if (getfilecon(orig_file,&passwd_context) < 0) {
+	    (void) fprintf(stderr,_("%s: Can't get context for %s"),progname,orig_file);
+	    pw_error(orig_file, 1, 1);
+	  }
+	  ret=setfilecon(tmp_file,passwd_context);
+	  freecon(passwd_context);
+	  if (ret!=0) {
+	    (void) fprintf(stderr,_("%s: Can't set context for %s"),progname,tmp_file);
+	    pw_error(tmp_file, 1, 1);
+	  }
+	}
+#endif
+
+	if (rename(tmp_file, orig_file) == -1) {
+		int errsv = errno;
+		fprintf(stderr,
+			_("%s: can't unlock %s: %s (your changes are still in %s)\n"),
+			progname, orig_file, strerror(errsv), tmp_file);
+		exit(EXIT_FAILURE);
+	}
+	unlink(tmp_file);
 }
 
 
@@ -211,17 +237,18 @@ pw_edit(int notsetuid) {
 		p = editor;
 
 	pid = fork();
-	if (pid < 0) {
-		(void)fprintf(stderr, _("%s: Cannot fork\n"), progname);
-		exit(1);
-	}
+	if (pid < 0)
+		err(EXIT_FAILURE, _("fork failed"));
+
 	if (!pid) {
 		if (notsetuid) {
 			(void)setgid(getgid());
 			(void)setuid(getuid());
 		}
 		execlp(editor, p, tmp_file, NULL);
-		_exit(1);
+
+		/* Shouldn't get here */
+		_exit(EXIT_FAILURE);
 	}
 	for (;;) {
 	    pid = waitpid(pid, &pstat, WUNTRACED);
@@ -238,27 +265,23 @@ pw_edit(int notsetuid) {
 }
 
 void
-pw_error(name, err, eval)
-	char *name;
-	int err, eval;
-{
-	int sverrno;
-
+pw_error(char *name, int err, int eval) {
 	if (err) {
-		sverrno = errno;
-		(void)fprintf(stderr, "%s: ", progname);
+		int sverrno = errno;
+
+		fprintf(stderr, "%s: ", progname);
 		if (name)
 			(void)fprintf(stderr, "%s: ", name);
-		(void)fprintf(stderr, "%s\n", strerror(sverrno));
+		fprintf(stderr, "%s\n", strerror(sverrno));
 	}
-	(void)fprintf(stderr,
+	fprintf(stderr,
 	    _("%s: %s unchanged\n"), progname, orig_file);
-	(void)unlink(tmp_file);
+	unlink(tmp_file);
 	exit(eval);
 }
 
 static void
-edit_file(void)
+edit_file(int is_shadow)
 {
 	struct stat begin, end;
 
@@ -267,69 +290,75 @@ edit_file(void)
 
 	if (stat(tmp_file, &begin))
 		pw_error(tmp_file, 1, 1);
+
 	pw_edit(0);
+
 	if (stat(tmp_file, &end))
 		pw_error(tmp_file, 1, 1);
 	if (begin.st_mtime == end.st_mtime) {
 		(void)fprintf(stderr, _("%s: no changes made\n"), progname);
 		pw_error((char *)NULL, 0, 0);
 	}
+	/* see pw_lock() where we create the file with mode 600 */
+	if (!is_shadow)
+		chmod(tmp_file, 0644);
+	else
+		chmod(tmp_file, 0400);
 	pw_unlock();
 }
 
-int main(int argc, char *argv[])
-{
+int main(int argc, char *argv[]) {
 
-  setlocale(LC_ALL, "");
-  bindtextdomain(PACKAGE, LOCALEDIR);
-  textdomain(PACKAGE);
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
 
-  bzero(tmp_file, FILENAMELEN);
-  progname = (rindex(argv[0], '/')) ? rindex(argv[0], '/') + 1 : argv[0];
-  if (!strcmp(progname, "vigr")) {
-    program = VIGR;
-    xstrncpy(orig_file, GROUP_FILE, sizeof(orig_file));
-    xstrncpy(tmp_file, GTMP_FILE, sizeof(tmp_file));
-    xstrncpy(tmptmp_file, GTMPTMP_FILE, sizeof(tmptmp_file));
-  } else {
-    program = VIPW;
-    xstrncpy(orig_file, PASSWD_FILE, sizeof(orig_file));
-    xstrncpy(tmp_file, PTMP_FILE, sizeof(tmp_file));
-    xstrncpy(tmptmp_file, PTMPTMP_FILE, sizeof(tmptmp_file));
-  }
+	memset(tmp_file, '\0', FILENAMELEN);
+	progname = (strrchr(argv[0], '/')) ? strrchr(argv[0], '/') + 1 : argv[0];
+	if (!strcmp(progname, "vigr")) {
+		program = VIGR;
+		xstrncpy(orig_file, GROUP_FILE, sizeof(orig_file));
+		xstrncpy(tmp_file, GTMP_FILE, sizeof(tmp_file));
+		xstrncpy(tmptmp_file, GTMPTMP_FILE, sizeof(tmptmp_file));
+	} else {
+		program = VIPW;
+		xstrncpy(orig_file, PASSWD_FILE, sizeof(orig_file));
+		xstrncpy(tmp_file, PTMP_FILE, sizeof(tmp_file));
+		xstrncpy(tmptmp_file, PTMPTMP_FILE, sizeof(tmptmp_file));
+	}
 
-  if ((argc > 1) && 
-      (!strcmp(argv[1], "-V") || !strcmp(argv[1], "--version"))) {
-    printf("%s\n", version_string);
-    exit(0);
-  }
+	if ((argc > 1) &&
+	    (!strcmp(argv[1], "-V") || !strcmp(argv[1], "--version"))) {
+		printf("%s\n", version_string);
+		exit(EXIT_SUCCESS);
+	}
 
-  edit_file();
+	edit_file(0);
 
-  if (program == VIGR) {
-    strncpy(orig_file, SGROUP_FILE, FILENAMELEN-1);
-    strncpy(tmp_file, SGTMP_FILE, FILENAMELEN-1);
-    strncpy(tmptmp_file, SGTMPTMP_FILE, FILENAMELEN-1);
-  } else {
-    strncpy(orig_file, SHADOW_FILE, FILENAMELEN-1);
-    strncpy(tmp_file, SPTMP_FILE, FILENAMELEN-1);
-    strncpy(tmptmp_file, SPTMPTMP_FILE, FILENAMELEN-1);
-  }
+	if (program == VIGR) {
+		strncpy(orig_file, SGROUP_FILE, FILENAMELEN-1);
+		strncpy(tmp_file, SGTMP_FILE, FILENAMELEN-1);
+		strncpy(tmptmp_file, SGTMPTMP_FILE, FILENAMELEN-1);
+	} else {
+		strncpy(orig_file, SHADOW_FILE, FILENAMELEN-1);
+		strncpy(tmp_file, SPTMP_FILE, FILENAMELEN-1);
+		strncpy(tmptmp_file, SPTMPTMP_FILE, FILENAMELEN-1);
+	}
 
-  if (!access(orig_file, X_OK)) {
-    char response[80];
+	if (access(orig_file, F_OK) == 0) {
+		char response[80];
 
-    printf((program == VIGR)
-	   ? _("You are using shadow groups on this system.\n")
-	   : _("You are using shadow passwords on this system.\n"));
-    printf(_("Would you like to edit %s now [y/n]? "), orig_file);
+		printf((program == VIGR)
+		       ? _("You are using shadow groups on this system.\n")
+		       : _("You are using shadow passwords on this system.\n"));
+		printf(_("Would you like to edit %s now [y/n]? "), orig_file);
 
-    /* EOF means no */
-    if (fgets(response, sizeof(response), stdin)) {
-	if (response[0] == 'y' || response[0] == 'Y')
-	    edit_file();
-    }
-  }
+		/* EOF means no */
+		if (fgets(response, sizeof(response), stdin)) {
+			if (response[0] == 'y' || response[0] == 'Y')
+				edit_file(1);
+		}
+	}
 
-  exit(0);
+	exit(EXIT_SUCCESS);
 }
