@@ -21,7 +21,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <err.h>
 #include <unistd.h>
 #include <getopt.h>
 #include <string.h>
@@ -30,14 +29,15 @@
 #include <sys/ioctl.h>
 #endif
 #include <assert.h>
+#include <poll.h>
 
-#define USE_UNSTABLE_LIBMOUNT_API
 #include <libmount.h>
 
 #include "pathnames.h"
 #include "nls.h"
 #include "c.h"
 #include "tt.h"
+#include "strutils.h"
 
 /* flags */
 enum {
@@ -48,6 +48,7 @@ enum {
 	FL_NOSWAPMATCH	= (1 << 6),
 	FL_NOFSROOT	= (1 << 7),
 	FL_SUBMOUNTS	= (1 << 8),
+	FL_POLL		= (1 << 9)
 };
 
 /* column IDs */
@@ -61,46 +62,58 @@ enum {
 	COL_LABEL,
 	COL_UUID,
 	COL_MAJMIN,
+	COL_ACTION,
+	COL_OLD_TARGET,
+	COL_OLD_OPTIONS,
 
-	__NCOLUMNS
+	FINDMNT_NCOLUMNS
 };
 
 /* column names */
 struct colinfo {
 	const char	*name;		/* header */
 	double		whint;		/* width hint (N < 1 is in percent of termwidth) */
-	int		truncate;	/* boolean */
+	int		flags;		/* tt flags */
+	const char      *help;		/* column description */
 	const char	*match;		/* pattern for match_func() */
 };
 
-/* columns descriptions */
-struct colinfo infos[__NCOLUMNS] = {
-	[COL_SOURCE]  = { "SOURCE",     0.25, FALSE },
-	[COL_TARGET]  = { "TARGET",     0.30, FALSE },
-	[COL_FSTYPE]  = { "FSTYPE",     0.10, TRUE },
-	[COL_OPTIONS] = { "OPTIONS",    0.10, TRUE },
-	[COL_VFS_OPTIONS] = { "VFS-OPTIONS", 0.20, TRUE },
-	[COL_FS_OPTIONS] = { "FS-OPTIONS", 0.10, TRUE },
-	[COL_LABEL]   = { "LABEL",      0.10, FALSE },
-	[COL_UUID]    = { "UUID",         36, FALSE },
-	[COL_MAJMIN] = { "MAJ:MIN",        6, FALSE },
+/* columns descriptions (don't use const, this is writable) */
+static struct colinfo infos[FINDMNT_NCOLUMNS] = {
+	[COL_SOURCE]       = { "SOURCE",       0.25, 0, N_("source device") },
+	[COL_TARGET]       = { "TARGET",       0.30, TT_FL_TREE, N_("mountpoint") },
+	[COL_FSTYPE]       = { "FSTYPE",       0.10, TT_FL_TRUNC, N_("filesystem type") },
+	[COL_OPTIONS]      = { "OPTIONS",      0.10, TT_FL_TRUNC, N_("all mount options") },
+	[COL_VFS_OPTIONS]  = { "VFS-OPTIONS",  0.20, TT_FL_TRUNC, N_("VFS specific mount options") },
+	[COL_FS_OPTIONS]   = { "FS-OPTIONS",   0.10, TT_FL_TRUNC, N_("FS specific mount options") },
+	[COL_LABEL]        = { "LABEL",        0.10, 0, N_("filesystem label") },
+	[COL_UUID]         = { "UUID",           36, 0, N_("filesystem UUID") },
+	[COL_MAJMIN]       = { "MAJ:MIN",         6, 0, N_("major:minor device number") },
+	[COL_ACTION]       = { "ACTION",         10, TT_FL_STRICTWIDTH, N_("action detected by --poll") },
+	[COL_OLD_OPTIONS]  = { "OLD-OPTIONS",  0.10, TT_FL_TRUNC, N_("old mount options saved by --poll") },
+	[COL_OLD_TARGET]   = { "OLD-TARGET",   0.30, 0, N_("old mountpoint saved by --poll") },
 };
 
 /* global flags */
-int flags;
-int tt_flags = 0;
+static int flags;
+static int tt_flags;
 
 /* array with IDs of enabled columns */
-int columns[__NCOLUMNS];
-int ncolumns;
+static int columns[FINDMNT_NCOLUMNS];
+static int ncolumns;
+
+/* poll actions (parsed --poll=<list> */
+#define FINDMNT_NACTIONS	4		/* mount, umount, move, remount */
+static int actions[FINDMNT_NACTIONS];
+static int nactions;
 
 /* libmount cache */
-struct libmnt_cache *cache;
+static struct libmnt_cache *cache;
 
 static int get_column_id(int num)
 {
 	assert(num < ncolumns);
-	assert(columns[num] < __NCOLUMNS);
+	assert(columns[num] < FINDMNT_NCOLUMNS);
 	return columns[num];
 }
 
@@ -111,7 +124,7 @@ static struct colinfo *get_column_info(int num)
 
 static const char *column_id_to_name(int id)
 {
-	assert(id < __NCOLUMNS);
+	assert(id < FINDMNT_NCOLUMNS);
 	return infos[id].name;
 }
 
@@ -125,23 +138,38 @@ static float get_column_whint(int num)
 	return get_column_info(num)->whint;
 }
 
-static int get_column_truncate(int num)
+static int get_column_flags(int num)
 {
-	return get_column_info(num)->truncate;
+	return get_column_info(num)->flags;
 }
 
 static const char *get_match(int id)
 {
-	assert(id < __NCOLUMNS);
+	assert(id < FINDMNT_NCOLUMNS);
 	return infos[id].match;
 }
 
 static void set_match(int id, const char *match)
 {
-	assert(id < __NCOLUMNS);
+	assert(id < FINDMNT_NCOLUMNS);
 	infos[id].match = match;
 }
 
+static int is_tabdiff_column(int id)
+{
+	assert(id < FINDMNT_NCOLUMNS);
+
+	switch(id) {
+	case COL_ACTION:
+	case COL_OLD_TARGET:
+	case COL_OLD_OPTIONS:
+		return 1;
+	default:
+		break;
+	}
+	return 0;
+}
+
 /*
  * "findmnt" without any filter
  */
@@ -153,6 +181,39 @@ static int is_listall_mode(void)
 		!get_match(COL_OPTIONS));
 }
 
+/*
+ * Returns 1 if the @act is in the --poll=<list>
+ */
+static int has_poll_action(int act)
+{
+	int i;
+
+	if (!nactions)
+		return 1;	/* all actions enabled */
+	for (i = 0; i < nactions; i++)
+		if (actions[i] == act)
+			return 1;
+	return 0;
+}
+
+static int poll_action_name_to_id(const char *name, size_t namesz)
+{
+	int id = -1;
+
+	if (strncasecmp(name, "move", namesz) == 0 && namesz == 4)
+		id = MNT_TABDIFF_MOVE;
+	else if (strncasecmp(name, "mount", namesz) == 0 && namesz == 5)
+		id = MNT_TABDIFF_MOUNT;
+	else if (strncasecmp(name, "umount", namesz) == 0 && namesz == 6)
+		id = MNT_TABDIFF_UMOUNT;
+	else if (strncasecmp(name, "remount", namesz) == 0 && namesz == 7)
+		id = MNT_TABDIFF_REMOUNT;
+	else
+		warnx(_("unknown action: %s"), name);
+
+	return id;
+}
+
 /*
  * findmnt --first-only <devname|TAG=|mountpoint>
  *
@@ -170,12 +231,12 @@ static int is_mount_compatible_mode(void)
 	return 1;			/* ok */
 }
 
-static void set_all_columns_truncate(int set)
+static void disable_columns_truncate(void)
 {
 	int i;
 
-	for (i = 0; i < __NCOLUMNS; i++)
-		infos[i].truncate = set;
+	for (i = 0; i < FINDMNT_NCOLUMNS; i++)
+		infos[i].flags &= ~TT_FL_TRUNC;
 }
 
 /*
@@ -185,7 +246,7 @@ static int column_name_to_id(const char *name, size_t namesz)
 {
 	int i;
 
-	for (i = 0; i < __NCOLUMNS; i++) {
+	for (i = 0; i < FINDMNT_NCOLUMNS; i++) {
 		const char *cn = column_id_to_name(i);
 
 		if (!strncasecmp(name, cn, namesz) && !*(cn + namesz))
@@ -250,7 +311,7 @@ static const char *get_data(struct libmnt_fs *fs, int num)
 		str = mnt_fs_get_fstype(fs);
 		break;
 	case COL_OPTIONS:
-		str = mnt_fs_strdup_options(fs);
+		str = mnt_fs_get_options(fs);
 		break;
 	case COL_VFS_OPTIONS:
 		str = mnt_fs_get_vfs_options(fs);
@@ -270,7 +331,7 @@ static const char *get_data(struct libmnt_fs *fs, int num)
 		if (devno) {
 			char *tmp;
 			int rc = 0;
-			if (tt_flags & TT_FL_RAW)
+			if ((tt_flags & TT_FL_RAW) || (tt_flags & TT_FL_EXPORT))
 				rc = asprintf(&tmp, "%u:%u",
 					      major(devno), minor(devno));
 			else
@@ -286,6 +347,53 @@ static const char *get_data(struct libmnt_fs *fs, int num)
 	return str;
 }
 
+static const char *get_tabdiff_data(struct libmnt_fs *old_fs,
+				    struct libmnt_fs *new_fs,
+				    int change,
+				    int num)
+{
+	const char *str = NULL;
+
+	switch (get_column_id(num)) {
+	case COL_ACTION:
+		switch (change) {
+		case MNT_TABDIFF_MOUNT:
+			str = _("mount");
+			break;
+		case MNT_TABDIFF_UMOUNT:
+			str = _("umount");
+			break;
+		case MNT_TABDIFF_REMOUNT:
+			str = _("remount");
+			break;
+		case MNT_TABDIFF_MOVE:
+			str = _("move");
+			break;
+		default:
+			str = _("unknown");
+			break;
+		}
+		break;
+	case COL_OLD_OPTIONS:
+		if (old_fs && (change == MNT_TABDIFF_REMOUNT ||
+			       change == MNT_TABDIFF_UMOUNT))
+			str = mnt_fs_get_options(old_fs);
+		break;
+	case COL_OLD_TARGET:
+		if (old_fs && (change == MNT_TABDIFF_MOVE ||
+			       change == MNT_TABDIFF_UMOUNT))
+			str = mnt_fs_get_target(old_fs);
+		break;
+	default:
+		if (new_fs)
+			str = get_data(new_fs, num);
+		else
+			str = get_data(old_fs, num);
+		break;
+	}
+	return str;
+}
+
 /* adds one line to the output @tab */
 static struct tt_line *add_line(struct tt *tt, struct libmnt_fs *fs,
 					struct tt_line *parent)
@@ -304,6 +412,23 @@ static struct tt_line *add_line(struct tt *tt, struct libmnt_fs *fs,
 	return line;
 }
 
+static struct tt_line *add_tabdiff_line(struct tt *tt, struct libmnt_fs *new_fs,
+			struct libmnt_fs *old_fs, int change)
+{
+	int i;
+	struct tt_line *line = tt_add_line(tt, NULL);
+
+	if (!line) {
+		warn(_("failed to add line to output"));
+		return NULL;
+	}
+	for (i = 0; i < ncolumns; i++)
+		tt_line_set_data(line, i,
+				get_tabdiff_data(old_fs, new_fs, change, i));
+
+	return line;
+}
+
 static int has_line(struct tt *tt, struct libmnt_fs *fs)
 {
 	struct list_head *p;
@@ -356,7 +481,8 @@ leave:
 }
 
 /* error callback */
-static int parser_errcb(struct libmnt_table *tb, const char *filename, int line)
+static int parser_errcb(struct libmnt_table *tb __attribute__ ((__unused__)),
+			const char *filename, int line)
 {
 	warn(_("%s: parse error at line %d"), filename, line);
 	return 0;
@@ -369,7 +495,7 @@ static struct libmnt_table *parse_tabfile(const char *path)
 	struct libmnt_table *tb = mnt_new_table();
 
 	if (!tb) {
-		warn(_("failed to initialize libmount tab"));
+		warn(_("failed to initialize libmount table"));
 		return NULL;
 	}
 
@@ -384,14 +510,15 @@ static struct libmnt_table *parse_tabfile(const char *path)
 
 	if (rc) {
 		mnt_free_table(tb);
-		warn(_("can't read: %s"), path);
+		warn(_("can't read %s"), path);
 		return NULL;
 	}
 	return tb;
 }
 
 /* filter function for libmount (mnt_table_find_next_fs()) */
-static int match_func(struct libmnt_fs *fs, void *data)
+static int match_func(struct libmnt_fs *fs,
+		      void *data __attribute__ ((__unused__)))
 {
 	int rc = flags & FL_INVERT ? 1 : 0;
 	const char *m;
@@ -499,6 +626,136 @@ done:
 	return rc;
 }
 
+static int poll_match(struct libmnt_fs *fs)
+{
+	int rc = match_func(fs, NULL);
+
+	if (rc == 0 && !(flags & FL_NOSWAPMATCH) &&
+	    get_match(COL_SOURCE) && !get_match(COL_TARGET)) {
+		/*
+		 * findmnt --poll /foo
+		 * The '/foo' source as well as target.
+		 */
+		const char *str = get_match(COL_SOURCE);
+
+		set_match(COL_TARGET, str);	/* swap */
+		set_match(COL_SOURCE, NULL);
+
+		rc = match_func(fs, NULL);
+
+		set_match(COL_TARGET, NULL);	/* restore */
+		set_match(COL_SOURCE, str);
+
+	}
+	return rc;
+}
+
+static int poll_table(struct libmnt_table *tb, const char *tabfile,
+		  int timeout, struct tt *tt, int direction)
+{
+	FILE *f;
+	int rc = -1;
+	struct libmnt_iter *itr = NULL;
+	struct libmnt_table *tb_new = NULL;
+	struct libmnt_tabdiff *diff = NULL;
+	struct pollfd fds[1];
+
+	tb_new = mnt_new_table();
+	if (!tb_new) {
+		warn(_("failed to initialize libmount table"));
+		goto done;
+	}
+
+	itr = mnt_new_iter(direction);
+	if (!itr) {
+		warn(_("failed to initialize libmount iterator"));
+		goto done;
+	}
+
+	diff = mnt_new_tabdiff();
+	if (!diff) {
+		warn(_("failed to initialize libmount tabdiff"));
+		goto done;
+	}
+
+	/* cache is unnecessary to detect changes */
+	mnt_table_set_cache(tb, NULL);
+	mnt_table_set_cache(tb_new, NULL);
+
+	f = fopen(tabfile, "r");
+	if (!f) {
+		warn(_("%s: open failed"), tabfile);
+		goto done;
+	}
+
+	mnt_table_set_parser_errcb(tb_new, parser_errcb);
+
+	fds[0].fd = fileno(f);
+	fds[0].events = POLLPRI;
+
+	while (1) {
+		struct libmnt_table *tmp;
+		struct libmnt_fs *old, *new;
+		int change, count;
+
+		count = poll(fds, 1, timeout);
+		if (count == 0)
+			break;	/* timeout */
+		if (count < 0) {
+			warn(_("poll() failed"));
+			goto done;
+		}
+
+		rewind(f);
+		rc = mnt_table_parse_stream(tb_new, f, tabfile);
+		if (!rc)
+			rc = mnt_diff_tables(diff, tb, tb_new);
+		if (rc < 0)
+			goto done;
+
+		count = 0;
+		mnt_reset_iter(itr, direction);
+		while(mnt_tabdiff_next_change(
+				diff, itr, &old, &new, &change) == 0) {
+
+			if (!has_poll_action(change))
+				continue;
+			if (!poll_match(new ? new : old))
+				continue;
+			count++;
+			rc = !add_tabdiff_line(tt, new, old, change);
+			if (rc)
+				goto done;
+			if (flags & FL_FIRSTONLY)
+				break;
+		}
+
+		if (count) {
+			rc = tt_print_table(tt);
+			if (rc)
+				goto done;
+		}
+
+		/* swap tables */
+		tmp = tb;
+		tb = tb_new;
+		tb_new = tmp;
+
+		tt_remove_lines(tt);
+		mnt_reset_table(tb_new);
+
+		if (count && (flags & FL_FIRSTONLY))
+			break;
+	}
+
+	rc = 0;
+done:
+	mnt_free_table(tb_new);
+	mnt_free_tabdiff(diff);
+	mnt_free_iter(itr);
+	return rc;
+}
+
 static void __attribute__((__noreturn__)) usage(FILE *out)
 {
 	int i;
@@ -515,38 +772,44 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	"\nOptions:\n"
 	" -s, --fstab            search in static table of filesystems\n"
 	" -m, --mtab             search in table of mounted filesystems\n"
-	" -k, --kernel           search in kernel table of mounted \n"
-        "                        filesystems (default)\n\n"
+	" -k, --kernel           search in kernel table of mounted\n"
+        "                          filesystems (default)\n\n"));
 
+	fprintf(out, _(
+	" -p, --poll[=<list>]    monitor changes in table of mounted filesystems\n"
+	" -w, --timeout <num>    upper limit in milliseconds that --poll will block\n\n"));
+
+	fprintf(out, _(
+	" -a, --ascii            use ASCII chars for tree formatting\n"
 	" -c, --canonicalize     canonicalize printed paths\n"
-	" -d, --direction <word> search direction - 'forward' or 'backward'\n"
-	" -e, --evaluate         print all TAGs (LABEL/UUID) evaluated\n"
-        " -f, --first-only       print the first found filesystem only\n"
-	" -h, --help             print this help\n"
-	" -i, --invert           invert sense of matching\n"
+	" -d, --direction <word> direction of search, 'forward' or 'backward'\n"
+	" -e, --evaluate         convert tags (LABEL/UUID) to device names\n"
+	" -f, --first-only       print the first found filesystem only\n"));
+
+	fprintf(out, _(
+	" -h, --help             display this help text and exit\n"
+	" -i, --invert           invert the sense of matching\n"
 	" -l, --list             use list format output\n"
-	" -n, --noheadings       don't print headings\n"
-	" -u, --notruncate       don't truncate text in columns\n"
+	" -n, --noheadings       don't print column headings\n"
+	" -u, --notruncate       don't truncate text in columns\n"));
+	fprintf(out, _(
 	" -O, --options <list>   limit the set of filesystems by mount options\n"
-	" -o, --output <list>    output columns\n"
-	" -r, --raw              use raw format output\n"
-	" -a, --ascii            use ascii chars for tree formatting\n"
-	" -t, --types <list>     limit the set of filesystem by FS types\n"
+	" -o, --output <list>    the output columns to be shown\n"
+	" -P, --pairs            use key=\"value\" output format\n"
+	" -r, --raw              use raw output format\n"
+	" -t, --types <list>     limit the set of filesystems by FS types\n"));
+	fprintf(out, _(
 	" -v, --nofsroot         don't print [/dir] for bind or btrfs mounts\n"
-	" -R, --submounts        print all submount for the matching filesystems\n"
-	" -S, --source <string>  device, LABEL= or UUID=device\n"
-	" -T, --target <string>  mountpoint\n\n"));
+	" -R, --submounts        print all submounts for the matching filesystems\n"
+	" -S, --source <string>  the device to mount (by name, LABEL= or UUID=)\n"
+	" -T, --target <string>  the mountpoint to use\n\n"));
 
 
 	fprintf(out, _("\nAvailable columns:\n"));
 
-	for (i = 0; i < __NCOLUMNS; i++) {
+	for (i = 0; i < FINDMNT_NCOLUMNS; i++)
+		fprintf(out, " %11s  %s\n", infos[i].name, _(infos[i].help));
 
-		fprintf(out, "  %-12s", infos[i].name);
-		if (i && !((i+1) % 3))
-			fputc('\n', out);
-	}
-	fputc('\n', out);
 
 	fprintf(out, _("\nFor more information see findmnt(1).\n"));
 
@@ -565,12 +828,12 @@ int main(int argc, char *argv[])
 	struct libmnt_table *tb = NULL;
 	char *tabfile = NULL;
 	int direction = MNT_ITER_FORWARD;
-	int i, c, rc = -1;
+	int i, c, rc = -1, timeout = -1;
 
 	/* table.h */
 	struct tt *tt = NULL;
 
-	struct option longopts[] = {
+	static const struct option longopts[] = {
 	    { "ascii",        0, 0, 'a' },
 	    { "canonicalize", 0, 0, 'c' },
 	    { "direction",    1, 0, 'd' },
@@ -586,33 +849,30 @@ int main(int argc, char *argv[])
 	    { "notruncate",   0, 0, 'u' },
 	    { "options",      1, 0, 'O' },
 	    { "output",       1, 0, 'o' },
+	    { "poll",         2, 0, 'p' },
+	    { "pairs",        0, 0, 'P' },
 	    { "raw",          0, 0, 'r' },
 	    { "types",        1, 0, 't' },
 	    { "fsroot",       0, 0, 'v' },
 	    { "submounts",    0, 0, 'R' },
 	    { "source",       1, 0, 'S' },
 	    { "target",       1, 0, 'T' },
+	    { "timeout",      1, 0, 'w' },
 
 	    { NULL,           0, 0, 0 }
 	};
 
-	assert(ARRAY_SIZE(columns) == __NCOLUMNS);
+	assert(ARRAY_SIZE(columns) == FINDMNT_NCOLUMNS);
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 
-	/* default enabled columns */
-	columns[ncolumns++] = COL_TARGET;
-	columns[ncolumns++] = COL_SOURCE;
-	columns[ncolumns++] = COL_FSTYPE;
-	columns[ncolumns++] = COL_OPTIONS;
-
 	/* default output format */
 	tt_flags |= TT_FL_TREE;
 
 	while ((c = getopt_long(argc, argv,
-				"acd:ehifo:O:klmnrst:uvRS:T:", longopts, NULL)) != -1) {
+				"acd:ehifo:O:p::Pklmnrst:uvRS:T:w:", longopts, NULL)) != -1) {
 		switch(c) {
 		case 'a':
 			tt_flags |= TT_FL_ASCII;
@@ -642,16 +902,33 @@ int main(int argc, char *argv[])
 			flags |= FL_FIRSTONLY;
 			break;
 		case 'u':
-			set_all_columns_truncate(FALSE);
+			disable_columns_truncate();
 			break;
 		case 'o':
-			if (tt_parse_columns_list(optarg, columns, &ncolumns,
-						column_name_to_id))
+			ncolumns = string_to_idarray(optarg,
+						columns, ARRAY_SIZE(columns),
+						column_name_to_id);
+			if (ncolumns < 0)
 				exit(EXIT_FAILURE);
 			break;
 		case 'O':
 			set_match(COL_OPTIONS, optarg);
 			break;
+		case 'p':
+			if (optarg) {
+				nactions = string_to_idarray(optarg,
+						actions, ARRAY_SIZE(actions),
+						poll_action_name_to_id);
+				if (nactions < 0)
+					exit(EXIT_FAILURE);
+			}
+			flags |= FL_POLL;
+			tt_flags &= ~TT_FL_TREE;
+			break;
+		case 'P':
+			tt_flags |= TT_FL_EXPORT;
+			tt_flags &= ~TT_FL_TREE;
+			break;
 		case 'm':		/* mtab */
 			if (tabfile)
 				errx_mutually_exclusive("--{fstab,mtab,kernel}");
@@ -677,8 +954,8 @@ int main(int argc, char *argv[])
 			tt_flags |= TT_FL_RAW;		/* enable raw */
 			break;
 		case 'l':
-			if (tt_flags & TT_FL_RAW)
-				errx_mutually_exclusive("--{raw,list}");
+			if ((tt_flags & TT_FL_RAW) && (tt_flags & TT_FL_EXPORT))
+				errx_mutually_exclusive("--{raw,list,pairs}");
 
 			tt_flags &= ~TT_FL_TREE; /* disable the default */
 			break;
@@ -699,12 +976,27 @@ int main(int argc, char *argv[])
 			set_match(COL_TARGET, optarg);
 			flags |= FL_NOSWAPMATCH;
 			break;
+		case 'w':
+			timeout = strtol_or_err(optarg,
+					_("failed to parse timeout"));
+			break;
 		default:
 			usage(stderr);
 			break;
 		}
 	}
 
+	/* default columns */
+	if (!ncolumns) {
+		if (flags & FL_POLL)
+			columns[ncolumns++] = COL_ACTION;
+
+		columns[ncolumns++] = COL_TARGET;
+		columns[ncolumns++] = COL_SOURCE;
+		columns[ncolumns++] = COL_FSTYPE;
+		columns[ncolumns++] = COL_OPTIONS;
+	}
+
 	if (!tabfile) {
 		tabfile = _PATH_PROC_MOUNTINFO;
 
@@ -770,11 +1062,17 @@ int main(int argc, char *argv[])
 	}
 
 	for (i = 0; i < ncolumns; i++) {
-		int fl = get_column_truncate(i) ? TT_FL_TRUNC : 0;
+		int fl = get_column_flags(i);
+		int id = get_column_id(i);
 
-		if (get_column_id(i) == COL_TARGET && (tt_flags & TT_FL_TREE))
-			fl |= TT_FL_TREE;
+		if (!(tt_flags & TT_FL_TREE))
+			fl &= ~TT_FL_TREE;
 
+		if (!(flags & FL_POLL) && is_tabdiff_column(id)) {
+			warnx(_("%s column is requested, but --poll "
+			       "is not enabled"), get_column_name(i));
+			goto leave;
+		}
 		if (!tt_define_column(tt, get_column_name(i),
 					get_column_whint(i), fl)) {
 			warn(_("failed to initialize output column"));
@@ -785,7 +1083,11 @@ int main(int argc, char *argv[])
 	/*
 	 * Fill in data to the output table
 	 */
-	if ((tt_flags & TT_FL_TREE) && is_listall_mode())
+	if (flags & FL_POLL)
+		/* poll mode */
+		rc = poll_table(tb, tabfile, timeout, tt, direction);
+
+	else if ((tt_flags & TT_FL_TREE) && is_listall_mode())
 		/* whole tree */
 		rc = create_treenode(tt, tb, NULL, NULL);
 	else
@@ -793,9 +1095,9 @@ int main(int argc, char *argv[])
 		rc = add_matching_lines(tb, tt, direction);
 
 	/*
-	 * Print the output table
+	 * Print the output table for non-poll modes
 	 */
-	if (!rc)
+	if (!rc && !(flags & FL_POLL))
 		tt_print_table(tt);
 leave:
 	tt_free_table(tt);
