@@ -6,9 +6,9 @@
  *   modify it under the terms of the gnu general public license.
  *   there is no warranty.
  *
- *   $Author: janl $
- *   $Revision: 1.5 $
- *   $Date: 1996/11/11 22:40:03 $
+ *   faith
+ *   1.2
+ *   1995/02/23 01:20:40
  *
  */
 
@@ -22,12 +22,13 @@
 #include <dirent.h>
 #include <ctype.h>
 #include <unistd.h>
+#include "kill.h"
 
 extern char *mybasename (char *);
 static char *parse_parens (char *buf);
 
-int *get_pids (char *process_name, int get_all)
-{
+int *
+get_pids (char *process_name, int get_all) {
     DIR *dir;
     struct dirent *ent;
     int status;
@@ -78,11 +79,13 @@ int *get_pids (char *process_name, int get_all)
 	    pids_size += 5;
 	    pids = (int *) realloc (pids, sizeof (int) * pids_size);
 	}
-	pids[num_pids++] = pid;
-	pids[num_pids] = -1;
+	if (pids) {
+		pids[num_pids++] = pid;
+		pids[num_pids] = -1;
+	}
     }
     closedir (dir);
-    return (pids);
+    return pids;
 }
 
 /*
@@ -112,7 +115,6 @@ static char *parse_parens (char *buf)
     return cp;
 }
 
-
 char *mybasename (char *path)
 {
     char *cp;
@@ -120,3 +122,4 @@ char *mybasename (char *path)
     cp = strrchr (path, '/');
     return (cp ? cp + 1 : path);
 }
+
