@@ -272,10 +272,15 @@ getmntfilebackward (const char *name, struct mntentchn *mcprev) {
 	mc0 = mtab_head();
 	if (!mcprev)
 		mcprev = mc0;
+
 	for (mc = mcprev->prev; mc && mc != mc0; mc = mc->prev)
-		if (streq(mc->m.mnt_dir, name) ||
-		    streq(mc->m.mnt_fsname, name))
+		if (streq(mc->m.mnt_dir, name))
 			return mc;
+
+	for (mc = mcprev->prev; mc && mc != mc0; mc = mc->prev)
+		if (streq(mc->m.mnt_fsname, name))
+			return mc;
+
 	return NULL;
 }
 
@@ -455,7 +460,13 @@ getfs_by_dir (const char *dir) {
 
 	cdir = canonicalize(dir);
 	for (mc = mc0->nxt; mc && mc != mc0; mc = mc->nxt) {
-		if (streq(mc->m.mnt_dir, cdir)) {
+		int ok = streq(mc->m.mnt_dir, cdir);
+		if (!ok) {
+			char *dr = canonicalize(mc->m.mnt_dir);
+			ok = dr ? streq(dr, cdir) : 0;
+			free(dr);
+		}
+		if (ok) {
 			free(cdir);
 			return mc;
 		}
@@ -574,7 +585,7 @@ handler (int sig) {
 }
 
 static void
-setlkw_timeout (int sig) {
+setlkw_timeout (int sig __attribute__ ((__unused__))) {
      /* nothing, fcntl will fail anyway */
 }
 
@@ -769,7 +780,7 @@ get_option(const char *optname, const char *src, size_t *len)
 		return NULL;
 
 	end = strchr(opt, ',');
-	sz = end ? end - opt : strlen(opt);
+	sz = end && end > opt ? (size_t) (end - opt) : strlen(opt);
 
 	if (len)
 		*len = sz;
