@@ -98,7 +98,7 @@ static int fake = 0;
  * returns: 0: no exec was done, 1: exec was done, status has result
  */
 static int
-check_special_umountprog(const char *spec, const char *node,
+check_special_umountprog(const char *node,
 			 const char *type, int *status) {
 	char umountprog[120];
 	struct stat statbuf;
@@ -263,7 +263,7 @@ static const char *chdir_to_parent(const char *node, char **resbuf)
    on a non-fatal error.  We lock/unlock around each umount.  */
 static int
 umount_one (const char *spec, const char *node, const char *type,
-	    const char *opts, struct mntentchn *mc) {
+	    struct mntentchn *mc) {
 	int umnt_err = 0;
 	int isroot;
 	int res = 0;
@@ -288,7 +288,7 @@ umount_one (const char *spec, const char *node, const char *type,
 	 * Call umount.TYPE for types that require a separate umount program.
 	 * All such special things must occur isolated in the types string.
 	 */
-	if (check_special_umountprog(spec, node, type, &status))
+	if (check_special_umountprog(node, type, &status))
 		return status;
 
 	block_signals(SIG_BLOCK);
@@ -455,13 +455,13 @@ umount_one_bw (const char *file, struct mntentchn *mc0) {
 	mc = mc0;
 	while (res && mc) {
 		res = umount_one(mc->m.mnt_fsname, mc->m.mnt_dir,
-				 mc->m.mnt_type, mc->m.mnt_opts, mc);
+				 mc->m.mnt_type, mc);
 		mc = getmntdirbackward(file, mc);
 	}
 	mc = mc0;
 	while (res && mc) {
 		res = umount_one(mc->m.mnt_fsname, mc->m.mnt_dir,
-				 mc->m.mnt_type, mc->m.mnt_opts, mc);
+				 mc->m.mnt_type, mc);
 		mc = getmntdevbackward(file, mc);
 	}
 	return res;
@@ -484,7 +484,7 @@ umount_all (char *types, char *test_opts) {
 	  if (matching_type (mc->m.mnt_type, types)
 	      && matching_opts (mc->m.mnt_opts, test_opts)) {
 	       errors |= umount_one (mc->m.mnt_fsname, mc->m.mnt_dir,
-				     mc->m.mnt_type, mc->m.mnt_opts, mc);
+				     mc->m.mnt_type, mc);
 	  }
      }
 
@@ -574,13 +574,33 @@ is_valid_loop(struct mntentchn *mc, struct mntentchn *fs)
 	return 0;
 }
 
+/*
+ * umount helper call based on {u,p}helper= mount option
+ */
+static int check_helper_umountprog(const char *node,
+				   const char *opts, const char *name,
+				   int *status)
+{
+	char *helper;
+
+	if (!external_allowed || !opts)
+		return 0;
+
+	helper = get_option_value(opts, name);
+	if (helper)
+		return check_special_umountprog(node, helper, status);
+
+	return 0;
+}
+
 static int
 umount_file (char *arg) {
 	struct mntentchn *mc, *fs;
 	const char *file, *options;
 	int fstab_has_user, fstab_has_users, fstab_has_owner, fstab_has_group;
-	int ok;
+	int ok, status = 0;
 	struct stat statbuf;
+	char *loopdev = NULL;
 
 	if (!*arg) {		/* "" would be expanded to `pwd` */
 		die(2, _("Cannot unmount \"\"\n"));
@@ -589,26 +609,7 @@ umount_file (char *arg) {
 
 	file = canonicalize(arg); /* mtab paths are canonicalized */
 
-	/* if file is a regular file, check if it is associated
-	 * with some loop device
-	 */
-	if (!stat(file, &statbuf) && S_ISREG(statbuf.st_mode)) {
-		char *loopdev = NULL;
-		switch (find_loopdev_by_backing_file(file, &loopdev)) {
-		case 0:
-			if (verbose)
-				printf(_("%s is associated with %s, trying to unmount it\n"),
-				       arg, loopdev);
-			file = loopdev;
-			break;
-		case 2:
-			if (verbose)
-				printf(_("%s is associated with more than one loop device: not unmounting\n"),
-				       arg);
-			break;
-		}
-	}
-
+try_loopdev:
 	if (verbose > 1)
 		printf(_("Trying to unmount %s\n"), file);
 
@@ -640,6 +641,35 @@ umount_file (char *arg) {
 	if (!mc && verbose)
 		printf(_("Could not find %s in mtab\n"), file);
 
+	/* not found in mtab - check if it is associated with some loop device
+	 * (only if it is a regular file)
+	 */
+	if (!mc && !loopdev && !stat(file, &statbuf) && S_ISREG(statbuf.st_mode)) {
+		switch (find_loopdev_by_backing_file(file, &loopdev)) {
+		case 0:
+			if (verbose)
+				printf(_("%s is associated with %s\n"),
+				       arg, loopdev);
+			file = loopdev;
+			goto try_loopdev;
+			break;
+		case 2:
+			if (verbose)
+				printf(_("%s is associated with more than one loop device: not unmounting\n"),
+				       arg);
+			break;
+		}
+	}
+
+	if (mc) {
+		/*
+		 * helper - umount helper (e.g. pam_mount)
+		 */
+		 if (check_helper_umountprog(arg, mc->m.mnt_opts,
+					    "helper=", &status))
+			return status;
+	}
+
 	if (restricted) {
 		char *mtab_user = NULL;
 
@@ -648,22 +678,11 @@ umount_file (char *arg) {
 			    _("umount: %s is not mounted (according to mtab)"),
 			    file);
 		/*
-		 * uhelper - unprivileged umount helper
-		 * -- external umount (for example HAL mounts)
+		 * uhelper - unprivileged umount helper (e.g. HAL/udisks mounts)
 		 */
-		if (external_allowed) {
-			char *uhelper = NULL;
-
-			if (mc->m.mnt_opts)
-				uhelper = get_option_value(mc->m.mnt_opts,
-							   "uhelper=");
-			if (uhelper) {
-				int status = 0;
-				if (check_special_umountprog(arg, arg,
-							uhelper, &status))
-					return status;
-			}
-		}
+		if (check_helper_umountprog(arg, mc->m.mnt_opts,
+					    "uhelper=", &status))
+			return status;
 
 		/* The 2.4 kernel will generally refuse to mount the same
 		   filesystem on the same mount point, but will accept NFS.
@@ -734,12 +753,13 @@ umount_file (char *arg) {
 			die (2, _("umount: only %s can unmount %s from %s"),
 			     mtab_user ? mtab_user : "root",
 			     fs->m.mnt_fsname, fs->m.mnt_dir);
+
 	}
 
 	if (mc)
 		return umount_one_bw (file, mc);
 	else
-		return umount_one (arg, arg, arg, arg, NULL);
+		return umount_one (arg, arg, arg, NULL);
 }
 
 int
