@@ -34,7 +34,7 @@
  */
 
  /*
-  * 1999-02-22 Arkadiusz Mikiewicz <misiek@misiek.eu.org>
+  * 1999-02-22 Arkadiusz Mikiewicz <misiek@pld.ORG.PL>
   * - added Native Language Support
   * Sun Mar 21 1999 - Arnaldo Carvalho de Melo <acme@conectiva.com.br>
   * - fixed strerr(errno) in gettext calls
@@ -50,15 +50,9 @@
 #include <fcntl.h>
 #include <errno.h>
 #include <signal.h>
-#include <linux/tty.h>
 #include <termios.h>
 
-#include "../defines.h"		/* for NEED_tqueue_h */
-#ifdef NEED_tqueue_h
-#include <linux/tqueue.h>	/* required for old kernels (for struct tq_struct) */
-				/* compilation errors on other kernels */
-#endif
-#include <linux/cyclades.h>
+#include "cyclades.h"
 
 #if 0
 #ifndef XMIT
@@ -69,6 +63,7 @@
 #endif
 #endif
 
+#include "xalloc.h"
 #include "nls.h"
 				/* Until it gets put in the kernel,
 				   toggle by hand. */
@@ -103,11 +98,9 @@ static char ***global_argv;
 static void
 summary(int sig) {
   struct cyclades_control *cc;
-
   int argc, local_optind;
   char **argv;
-
-  int i,j;
+  int i;
 
   argc = global_argc;
   argv = *global_argv;
@@ -115,7 +108,6 @@ summary(int sig) {
 
   if (sig > 0) {
     for(i = local_optind; i < argc; i ++) {
-      j = i - local_optind;
       cc = &cmon[cmon_index];
       fprintf(stderr, _("File %s, For threshold value %lu, Maximum characters in fifo were %d,\nand the maximum transfer rate in characters/second was %f\n"), 
 	      argv[i],
@@ -184,8 +176,7 @@ int main(int argc, char *argv[]) {
   bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
 
-  while (EOF != (i = getopt(argc, argv, 
-			    "qs:S:t:T:gGi:"))) {
+  while ((i = getopt(argc, argv, "qs:S:t:T:gGi:")) != -1) {
     switch (i) {
     case 'q':
       query = 1;
@@ -250,7 +241,7 @@ int main(int argc, char *argv[]) {
   global_optind = optind;	/* For signal routine. */
 
   if (set || set_def) {
-    for(i = optind;i < argc;i ++) {
+    for(i = optind; i < argc; i++) {
       file = open(argv[i],O_RDONLY);
       if(file == -1) {
         int errsv = errno;
@@ -265,10 +256,11 @@ int main(int argc, char *argv[]) {
 		argv[i],set?set_val:set_def_val,strerror(errsv));
 	exit(1);
       }
+      close(file);
     }
   }
   if (set_time || set_def_time) {
-    for(i = optind;i < argc;i ++) {
+    for(i = optind; i < argc; i++) {
       file = open(argv[i],O_RDONLY);
       if(file == -1) {
         int errsv = errno;
@@ -283,11 +275,12 @@ int main(int argc, char *argv[]) {
 		argv[i],set_time?set_time_val:set_def_time_val,strerror(errsv));
 	exit(1);
       }
+      close(file);
     }
   }
 
   if (get || get_def) {
-    for(i = optind;i < argc;i ++) {
+    for(i = optind; i < argc; i++) {
       file = open(argv[i],O_RDONLY);
       if(file == -1) {
         int errsv = errno;
@@ -306,11 +299,13 @@ int main(int argc, char *argv[]) {
 		argv[i],strerror(errsv));
 	exit(1);
       }
-      printf(_("%s: %ld %s threshold and %ld %s timeout\n"),
-	     argv[i], threshold_value, 
-	     get?_("current"):_("default"),
-	     timeout_value,
-	     get?_("current"):_("default"));
+      close(file);
+      if (get)
+	      printf(_("%s: %ld current threshold and %ld current timeout\n"),
+		     argv[i], threshold_value, timeout_value);
+      else
+	      printf(_("%s: %ld default threshold and %ld default timeout\n"),
+		     argv[i], threshold_value, timeout_value);
     }
   }
 
@@ -318,12 +313,8 @@ int main(int argc, char *argv[]) {
 
   /* query stuff after this line */
   
-  cmon = (struct cyclades_control *) malloc(sizeof (struct cyclades_control)
-					    * numfiles);
-  if(!cmon) {
-    perror(_("malloc failed"));
-    exit(1);
-  }
+  cmon = xmalloc(sizeof(struct cyclades_control) * numfiles);
+
   if(signal(SIGINT, summary)||
      signal(SIGQUIT, summary)||
      signal(SIGTERM, summary)) {
@@ -413,14 +404,14 @@ int main(int argc, char *argv[]) {
 	if(xmit_rate > cmon[cmon_index].maxxmit)
           cmon[cmon_index].maxxmit = xmit_rate;
 #endif
-	if(cywork.char_max > cmon[cmon_index].maxmax) 
+	if(cmon[cmon_index].maxmax < 0 ||
+	   cywork.char_max > (unsigned long) cmon[cmon_index].maxmax)
 	  cmon[cmon_index].maxmax = cywork.char_max;
       }
 
 #ifdef XMIT
-      printf(_("%s: %lu ints, %lu/%lu chars; ")
-	     _("fifo: %lu thresh, %lu tmout, ")
-	     _("%lu max, %lu now\n"),
+      printf(_("%s: %lu ints, %lu/%lu chars; fifo: %lu thresh, %lu tmout, "
+	       "%lu max, %lu now\n"),
 	     argv[i],
 	     cywork.int_count,cywork.char_count,cywork.send_count,
 	     threshold_value,timeout_value,
@@ -430,7 +421,8 @@ int main(int argc, char *argv[]) {
 	     xfer_rate,
 	     xmit_rate);
 #else
-      printf(_("%s: %lu ints, %lu chars; fifo: %lu thresh, %lu tmout, %lu max, %lu now\n"),
+      printf(_("%s: %lu ints, %lu chars; fifo: %lu thresh, %lu tmout, "
+	       "%lu max, %lu now\n"),
 	     argv[i],
 	     cywork.int_count,cywork.char_count,
 	     threshold_value,timeout_value,
