@@ -49,6 +49,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
+
+#include "xalloc.h"
 #include "nls.h"
 
 #define S_LEN 128
@@ -56,26 +58,10 @@
 static char *prgname;
 
 /* These are the defaults */
-static char defaultmap[]="/usr/src/linux/System.map";
+static char defaultmap[]="/boot/System.map";
 static char defaultpro[]="/proc/profile";
 static char optstring[]="M:m:np:itvarVbs";
 
-static void *
-xmalloc (size_t size) {
-	void *t;
-
-	if (size == 0)
-		return NULL;
-
-	t = malloc (size);
-	if (t == NULL) {
-		fprintf(stderr, _("out of memory"));
-		exit(1);
-	}
-
-	return t;
-}
-
 static FILE *
 myopen(char *name, char *mode, int *flag) {
 	int len = strlen(name);
@@ -136,7 +122,7 @@ main(int argc, char **argv) {
 	FILE *map;
 	int proFd;
 	char *mapFile, *proFile, *mult=0;
-	unsigned long len=0, indx=1;
+	size_t len=0, indx=1;
 	unsigned long long add0=0;
 	unsigned int step;
 	unsigned int *buf, total, fn_len;
@@ -144,6 +130,7 @@ main(int argc, char **argv) {
 	char fn_name[S_LEN], next_name[S_LEN];   /* current and next name */
 	char mode[8];
 	int c;
+	ssize_t rc;
 	int optAll=0, optInfo=0, optReset=0, optVerbose=0, optNative=0;
 	int optBins=0, optSub=0;
 	char mapline[S_LEN];
@@ -194,8 +181,8 @@ main(int argc, char **argv) {
 			optVerbose++;
 			break;
 		case 'V':
-			printf(_("%s version %s\n"), prgname,
-			       UTIL_LINUX_VERSION);
+			printf(_("%s (%s)\n"), prgname,
+			       PACKAGE_STRING);
 			exit(0);
 		default:
 			usage();
@@ -224,7 +211,7 @@ main(int argc, char **argv) {
 			exit(1);
 		}
 		if (write(fd, &multiplier, to_write) != to_write) {
-			fprintf(stderr, "readprofile: error writing %s: %s\n",
+			fprintf(stderr, _("readprofile: error writing %s: %s\n"),
 				defaultpro, strerror(errno));
 			exit(1);
 		}
@@ -242,12 +229,10 @@ main(int argc, char **argv) {
 		exit(1);
 	}
 
-	if (!(buf=malloc(len))) {
-		fprintf(stderr,"%s: malloc(): %s\n", prgname, strerror(errno));
-		exit(1);
-	}
+	buf = xmalloc(len);
 
-	if (read(proFd,buf,len) != len) {
+	rc = read(proFd,buf,len);
+	if (rc < 0 || (size_t) rc != len) {
 		fprintf(stderr,"%s: %s: %s\n",prgname,proFile,strerror(errno));
 		exit(1);
 	}
@@ -255,8 +240,9 @@ main(int argc, char **argv) {
 
 	if (!optNative) {
 		int entries = len/sizeof(*buf);
-		int big = 0,small = 0,i;
+		int big = 0,small = 0;
 		unsigned *p;
+		size_t i;
 
 		for (p = buf+1; p < buf+entries; p++) {
 			if (*p & ~0U << (sizeof(*buf)*4))
@@ -265,8 +251,8 @@ main(int argc, char **argv) {
 				small++;
 		}
 		if (big > small) {
-			fprintf(stderr,"Assuming reversed byte order. "
-				"Use -n to force native byte order.\n");
+			fprintf(stderr,_("Assuming reversed byte order. "
+				"Use -n to force native byte order.\n"));
 			for (p = buf; p < buf+entries; p++)
 				for (i = 0; i < sizeof(*buf)/2; i++) {
 					unsigned char *b = (unsigned char *) p;
@@ -306,7 +292,8 @@ main(int argc, char **argv) {
 				prgname, mapFile, maplineno);
 			exit(1);
 		}
-		if (!strcmp(fn_name,"_stext")) /* only elf works like this */ {
+		/* only elf works like this */
+		if (!strcmp(fn_name,"_stext") || !strcmp(fn_name,"__stext")) {
 			add0 = fn_add;
 			break;
 		}
@@ -324,6 +311,7 @@ main(int argc, char **argv) {
 	 */
 	while (fgets(mapline,S_LEN,map)) {
 		unsigned int this=0;
+		int done = 0;
 
 		if (sscanf(mapline,"%llx %s %s",&next_add,mode,next_name)!=3) {
 			fprintf(stderr,_("%s: %s(%i): wrong map line\n"),
@@ -332,12 +320,21 @@ main(int argc, char **argv) {
 		}
 		header_printed = 0;
 
-		/* ignore any LEADING (before a '[tT]' symbol is found)
-		   Absolute symbols */
-		if ((*mode == 'A' || *mode == '?') && total == 0) continue;
-		if (*mode != 'T' && *mode != 't' &&
-		    *mode != 'W' && *mode != 'w')
-			break;	/* only text is profiled */
+		/* the kernel only profiles up to _etext */
+		if (!strcmp(next_name, "_etext") ||
+		    !strcmp(next_name, "__etext"))
+			done = 1;
+		else {
+			/* ignore any LEADING (before a '[tT]' symbol is found)
+			   Absolute symbols and __init_end because some
+			   architectures place it before .text section */
+			if ((*mode == 'A' || *mode == '?')
+			    && (total == 0 || !strcmp(next_name, "__init_end")))
+				continue;
+			if (*mode != 'T' && *mode != 't' &&
+			    *mode != 'W' && *mode != 'w')
+				break;	/* only text is profiled */
+		}
 
 		if (indx >= len / sizeof(*buf)) {
 			fprintf(stderr, _("%s: profile address out of range. "
@@ -387,7 +384,13 @@ main(int argc, char **argv) {
 		strcpy(fn_name,next_name);
 
 		maplineno++;
+		if (done)
+			break;
 	}
+
+	/* clock ticks, out of kernel text - probably modules */
+	printf("%6i %s\n", buf[len/sizeof(*buf)-1], "*unknown*");
+
 	/* trailer */
 	if (optVerbose)
 		printf("%016x %-40s %6i %8.4f\n",
