@@ -31,36 +31,38 @@
  * SUCH DAMAGE.
  */
 
-#ifndef lint
-char copyright[] =
-"@(#) Copyright (c) 1989 The Regents of the University of California.\n\
- All rights reserved.\n";
-#endif /* not lint */
-
-#ifndef lint
-static char sccsid[] = "@(#)hexdump.c	5.5 (Berkeley) 6/1/90";
-#endif /* not lint */
+ /* 1999-02-22 Arkadiusz Mikiewicz <misiek@pld.ORG.PL>
+  * - added Native Language Support
+  */
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
 #include "hexdump.h"
 
+#include "nls.h"
+#include "c.h"
+
 FS *fshead;				/* head of format strings */
 int blocksize;				/* data block size */
 int exitval;				/* final exit value */
 int length = -1;			/* max bytes to read */
 
-int main(argc, argv)
-	int argc;
-	char **argv;
+int main(int argc, char **argv)
 {
-	register FS *tfs;
-	char *p, *rindex();
+	FS *tfs;
+	char *p;
 
-	if (!(p = rindex(argv[0], 'o')) || strcmp(p, "od"))
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
+
+	if (!(p = strrchr(argv[0], 'o')) || strcmp(p, "od")) {
 		newsyntax(argc, &argv);
-	else
-		oldsyntax(argc, &argv);
+	} else
+		errx(EXIT_FAILURE, _("calling hexdump as od has been deprecated "
+				     "in favour to GNU coreutils od."));
 
 	/* figure out the data block size */
 	for (blocksize = 0, tfs = fshead; tfs; tfs = tfs->nextfs) {
